"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Case = require("case");
var morgan = require("morgan");
var os = require("os");
var DEFAULT_HOSTNAME = os.hostname() || process.env.HOSTNAME || 'unknown';
var DEFAULT_LOGGER = global.console;
var DEFAULT_LEVEL = 'info';
/**
 * Boostraps an Express-compatible middleware using Morgan.
 * Logs will be routed through :logger via the specified :level.
 * Tokens can be added using :additionalTokens and the :hostname
 * will be as specified.
 *
 * @param {CreateLoggerParameters} params
 * @param {Tokenizer[]} params.additionalTokens
 * @param {String} params.hostname
 * @param {LoggerInterface} params.logger
 * @param {String} params.level
 *
 * @throws {Error} when the :level` cannot be found in the :logger
 *
 * @return {express.RequestHandler}
 */
function createMiddleware(_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.additionalTokens, additionalTokens = _c === void 0 ? [] : _c, _d = _b.hostname, hostname = _d === void 0 ? DEFAULT_HOSTNAME : _d, _e = _b.logger, logger = _e === void 0 ? DEFAULT_LOGGER : _e, _f = _b.level, level = _f === void 0 ? DEFAULT_LEVEL : _f;
    if (typeof logger[level] !== 'function') {
        // tslint:disable-next-line max-line-length
        throw new Error("The specified level, \"" + level + "\", could not be found in the provided :logger.");
    }
    morgan.token('hostname', function () { return hostname; });
    additionalTokens.forEach(function (token) {
        morgan.token(token.id, token.fn);
    });
    return morgan(function (tokens, req, res) {
        var message = {
            contentLength: tokens['res'](req, res, 'content-length'),
            httpVersion: tokens['http-version'](req, res),
            level: 'access',
            method: tokens['method'](req, res),
            referrer: tokens['referrer'](req, res),
            remoteAddress: tokens['remote-addr'](req, res),
            remoteHostname: req['hostname'],
            responseTimeMs: tokens['response-time'](req, res),
            serverHostname: tokens['hostname'](req, res),
            status: tokens['status'](req, res),
            time: tokens['date'](req, res, 'iso'),
            url: tokens['url'](req, res),
            userAgent: tokens['user-agent'](req, res),
        };
        additionalTokens.forEach(function (token) {
            var messageKey = Case.camel(token.id);
            if (!message[messageKey]) {
                message[messageKey] = tokens[token.id](req, res);
            }
        });
        return JSON.stringify(message);
    }, {
        stream: {
            write: function (_message) {
                var message = JSON.parse(_message);
                logger[level](message);
            },
        },
    });
}
exports.createMiddleware = createMiddleware;
function getZipkinTokenizers() {
    return [
        {
            fn: function (req) { return req.context.traceId; },
            id: 'trace-id',
        },
        {
            fn: function (req) { return req.context.spanId; },
            id: 'span-id',
        },
        {
            fn: function (req) { return req.context.parentId; },
            id: 'parent-span-id',
        },
        {
            fn: function (req) { return req.context.sampled; },
            id: 'sampled',
        },
    ];
}
exports.getZipkinTokenizers = getZipkinTokenizers;
//# sourceMappingURL=index.js.map