"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var supertest = require("supertest");
var superagent = require("superagent");
var chai = require("chai");
var sinon = require("sinon");
var sinonChai = require("sinon-chai");
var _1 = require("./");
chai.use(sinonChai);
var expect = chai.expect;
describe('@joeir/logger-request', function () {
    var loggerMock = {};
    var server;
    var instance;
    before(function () {
        loggerMock.mock = sinon.spy();
        loggerMock.info = sinon.spy();
    });
    beforeEach(function () {
        server = express();
    });
    afterEach(function () {
        if (instance) {
            instance.close();
        }
        Object.keys(loggerMock).forEach(function (key) {
            loggerMock[key].resetHistory();
        });
    });
    it('has the correct log paremeters', function (done) {
        server.use(_1.createMiddleware({
            logger: loggerMock,
        }));
        server.get('/', function (_req, res) {
            res.json('ok');
        });
        instance = server.listen(function () {
            superagent
                .get("http://localhost:" + instance.address()['port'])
                .end(function (err, response) {
                expect(loggerMock.info).to.be.calledOnce;
                expect(loggerMock.info).to.be.calledWith(sinon.match(function (_a) {
                    var level = _a.level, method = _a.method, url = _a.url, status = _a.status, contentLength = _a.contentLength, responseTimeMs = _a.responseTimeMs, httpVersion = _a.httpVersion, userAgent = _a.userAgent;
                    return (level === 'access'
                        && method === 'GET'
                        && url === '/'
                        && status === '200'
                        && contentLength === '4'
                        && httpVersion === '1.1'
                        && !isNaN(Math.floor(responseTimeMs))
                        && (userAgent.match(/^node-superagent/gi) !== null));
                }));
                instance.close();
                done();
            });
        });
    });
    context('integration', function () {
        describe('basic', function () {
            it('works', function () {
                expect(function () { return _1.createMiddleware(); }).to.not.throw();
                server.use(_1.createMiddleware({
                    logger: loggerMock,
                }));
                server.get('/', function (_req, res) {
                    res.json('ok');
                });
                return supertest(server)
                    .get('/')
                    .expect(200)
                    .then(function () {
                    [
                        function (log) { return log.level === 'access'; },
                        function (log) { return log.method === 'GET'; },
                        function (log) { return log.url === '/'; },
                        function (log) { return !isNaN(Math.floor(log.status)); },
                        function (log) { return !isNaN(Math.floor(log.contentLength)); },
                        function (log) { return !isNaN(Math.floor(log.responseTimeMs)); },
                        function (log) { return !isNaN(Math.floor(log.httpVersion)); },
                        function (log) { return typeof log.remoteHostname !== 'undefined'; },
                        function (log) { return typeof log.serverHostname !== 'undefined'; },
                        function (log) { return typeof log.time !== 'undefined'; },
                        function (log) { return typeof log.userAgent !== 'undefined'; },
                    ].forEach(function (supposedMatch) {
                        expect(loggerMock.info).to.be.calledWith(sinon.match(supposedMatch));
                    });
                });
            });
        });
        describe('extensions', function () {
            it('works', function () {
                var arbitraryNumber = Math.floor(Math.random() * 100000);
                var mockHostname = '__test_hostname';
                server.use(function (req, _res, next) {
                    req.arbitrary = arbitraryNumber;
                    next();
                });
                server.use(_1.createMiddleware({
                    additionalTokens: [
                        {
                            id: 'arbitrary-log',
                            fn: function (req, res) { return req.arbitrary; },
                        },
                    ],
                    hostname: mockHostname,
                    logger: loggerMock,
                    level: 'mock',
                }));
                server.get('/', function (req, res) {
                    expect(req.arbitrary).to.exist;
                    expect(req.arbitrary).to.equal(arbitraryNumber);
                    res.json('ok');
                });
                return supertest(server)
                    .get('/')
                    .expect(200)
                    .then(function (response) {
                    expect(response.body).to.equal('ok');
                    expect(loggerMock.mock).to.be.calledOnce;
                    expect(loggerMock.mock).to.be.calledWith(sinon.match(function (val) {
                        return val.arbitraryLog === arbitraryNumber;
                    }));
                    expect(loggerMock.mock).to.be.calledWith(sinon.match(function (val) {
                        return val.serverHostname === mockHostname;
                    }));
                });
            });
        });
    });
});
//# sourceMappingURL=index.test.js.map